#!/bin/bash
#
# Convenience script for flashing Ubuntu phones
# and making common configurations.
#
# recovery images:  (see 'urls' file)

WIFI=no
MEDIA=no
FEED=
CUSTOM=no
SPANISH=no
EXTRA=
PASSWD=0000
DEVMODE="--developer-mode --password=$PASSWD"
ADB=yes
GU=no
ASK=yes
WIZARD=yes
EDGEINTRO=yes
CHANNELOVERRIDE=
APTUPDATE=no
BEEP=yes
DEVTAR=no

for arg in $* ; do
  case "$arg" in
    --rtm)
      FEED=rtm
      shift
      ;;
    --custom)
      CUSTOM=yes
      shift
      ;;
    --spanish)
      SPANISH=yes
      echo "Spanish channul URL not known; ignoring option."
      shift
      ;;
    --network)
      WIFI=yes
      ADB=yes
      shift
      ;;
    --media)
      MEDIA=yes
      ADB=yes
      shift
      ;;
    --nodm)
      DEVMODE=""
      ADB=no
      shift
      ;;
    --gu)
      GU=yes
      ADB=yes
      shift
      ;;
    --yes)
      ASK=no
      shift
      ;;
    --nowizard)
      WIZARD=no
      shift
      ;;
    --nointro)
      EDGEINTRO=no
      shift
      ;;
    --apt-update)
      APTUPDATE=yes
      shift
      ;;
    --channel)
      shift
      CHANNELOVERRIDE=$1
      shift
      ;;
    --nobeep)
      BEEP=no
      shift
      ;;
    --devtarball)
      shift
      DEVTAR="$1"
      shift
      ;;
    --device)
      shift
      MODEL=$1
      shift
      ;;
    --rev)
      shift
      REVISION=$1
      shift
      ;;
    *)
      #echo "Arg: $arg"
      ;;
esac
done
# Positional arguments
if [ x"$MODEL" = x ]; then MODEL=$1 ; shift ; fi
if [ x"$REVISION" = x ]; then REVISION=$1 ; shift ; fi

REVARGS=
SERVER=
CHANNEL=ubuntu-touch/devel-proposed
FASTBOOT=no
RECOVERY=

# Figure out the channel and other options based on the phone type
echo "Model: $MODEL"
if [ x"$REVISION" != x ]; then
  echo "Revision: $REVISION"
  REVARGS="--revision=$REVISION"
fi

if [ x"$DEVTAR" = x"no" ]; then
  DEVTAR=""
else
  DEVTAR="--device-tarball $DEVTAR --run-script ./disable-gpg"
fi

# FIXME: Rewrite all this to use a separate file with device channels
# FIXME: Remove everything related to rtm and other obsolete options
if [ x"$MODEL" = x"mako" ]; then
  CHANNEL=ubuntu-touch/devel-proposed
  FASTBOOT=no
  if [ x"$FEED" = xrtm ]; then
    if [ x"$CUSTOM" == xyes ]; then
      CHANNEL=ubuntu-touch/ubuntu-rtm/14.09-proposed-customized
    else
      CHANNEL=ubuntu-touch/ubuntu-rtm/14.09-proposed
    fi
  fi
fi

if [ x"$MODEL" = xkrillin ]; then
  #CHANNEL=ubuntu-touch/devel-proposed/krillin.en
  CHANNEL=ubuntu-touch/rc-proposed/bq-aquaris.en
  FASTBOOT=yes
  RECOVERY='--recovery-image krillin/recovery.img'

  if [ x"$FEED" = xrtm ]; then
    if [ x"$CUSTOM" == xyes ]; then
      CHANNEL=ubuntu-touch/ubuntu-rtm/14.09-proposed-customized
    else
      CHANNEL=ubuntu-touch/ubuntu-rtm/14.09-proposed
      #CHANNEL=ubuntu-touch/ubuntu-rtm/14.09-factory-proposed
      #CHANNEL=ubuntu-touch/rc/bq-aquaris.en # release candidates
      #CHANNEL=ubuntu-touch/stable/bq-aquaris.en # released
      #CHANNEL=ubuntu-touch/ubuntu-rtm/devel-proposed
    fi
    echo "Feed: $FEED"
  fi
fi

if [ x"$MODEL" = xm75  -o  x"$MODEL" = xarale ]; then
  if [ x"$GU" = x"yes" ]; then
    GU=21
  fi

  # Capomastro
  #MODEL=arale
  #SERVER="--server https://sis.capomastro.canonical.com"
  #CHANNEL=ubuntu-touch/rc-proposed/meizu
  #FASTBOOT=yes
  ##RECOVERY='--recovery-image arale/recovery.img'

  # Public server
  MODEL=arale
  CHANNEL=ubuntu-touch/rc-proposed/meizu.en
  FASTBOOT=yes
  RECOVERY='--recovery-image arale/recovery.img'
fi

# FIXME: unify mako sections
if [ x"$MODEL" = xmako ]; then
  CHANNEL=ubuntu-touch/rc-proposed/bq-aquaris.en
  #FASTBOOT=yes
  #RECOVERY='--recovery-image krillin/recovery.img'
fi

if [ x"$CHANNELOVERRIDE" != x ]; then
  CHANNEL=$CHANNELOVERRIDE
fi

# Summarize what we're going to do
echo "Server: $SERVER"
echo "Channel: $CHANNEL"
echo "Feed: $FEED"
echo "Wifi: $WIFI"
echo "Media: $MEDIA"
echo "Fastboot: $FASTBOOT"
echo "Recovery: $RECOVERY"
echo ubuntu-device-flash $SERVER $REVARGS query --channel=$CHANNEL --device $MODEL --show-image
ubuntu-device-flash $SERVER $REVARGS query --channel=$CHANNEL --device $MODEL --show-image

# Ask the user to confirm, maybe
if [ x"$ASK" = x"yes" ]; then
  echo -n "Press enter to continue, or ^C to exit: "
  read
fi

function rootcmd {
  adb shell "echo $PASSWD | sudo -S $*"
}

# Handle fastboot mode and fastboot recovery mode
if [ x"$FASTBOOT_RECOVERY" != x ]; then
  echo 'Rebooting to bootloader/fastboot'
  adb reboot bootloader
  sleep 10
  echo "Flashing recovery image $FASTBOOT_RECOVERY"
  fastboot flash recovery $FASTBOOT_RECOVERY
  echo "Rebooting into recovery"
  fastboot boot $FASTBOOT_RECOVERY
  sleep 10
  FASTBOOT=no
elif [ x"$FASTBOOT" = xyes ]; then
  EXTRA="--bootstrap"
  echo 'Rebooting to bootloader/fastboot'
  adb reboot bootloader
  sleep 10
fi

echo 'flashing device'
echo ubuntu-device-flash $SERVER $REVARGS touch --channel=$CHANNEL --device $MODEL $DEVMODE $RECOVERY --wipe $DEVTAR $EXTRA
ubuntu-device-flash $SERVER $REVARGS touch --channel=$CHANNEL --device $MODEL $DEVMODE $RECOVERY --wipe $DEVTAR $EXTRA

echo 'waiting for device to boot'
adb wait-for-device
echo 'waiting a few seconds for boot'
sleep 5

# Try to ensure adb is going to work
# (may require a human to help)
if [ x$ADB = xyes ]; then
  # make sure adb shell works
  FAILED=no
  while ! adb shell echo success | grep success > /dev/null ; do
    echo "Please unlock the screen manually to enable developer mode"
    FAILED=yes
    sleep 1
  done
  rootcmd touch /userdata/.adb_onlock
  # alternate method:  (if $FAILED)
  # - adb reboot bootloader
  # - fastboot boot device/recovery.img
  # - make sure /data is mounted
  # - touch /data/.adb_onlock
  # - adb reboot
else
  if [ x"$BEEP" = x"yes" ]; then beep ; fi
  exit 0
fi

# Override the widget sizes, maybe
if [ x"$GU" != x  -a  x"$GU" != xyes  -a  x"$GU" != xno ]; then
  echo 'Setting better GU value'
  adb shell 'printf "start on starting dbus\nexec initctl set-env --global GRID_UNIT_PX='$GU'\n" > ~/.config/upstart/gu.conf'
fi

# Set up wifi!
# (configure this via phablet-network)
# (or uncomment the other lines here)
if [ x"$WIFI" = xyes ]; then
  echo 'connecting to the network'
  #WIFINAME=$(grep ^name $HOME/.wifi_auth | cut -d '=' -f 2)
  #WIFIPASSWD=$(grep ^passwd $HOME/.wifi_auth | cut -d '=' -f 2)
  #rootcmd nmcli d wifi connect $WIFINAME password $WIFIPASSWD iface wlan0
  phablet-network  # woot, I finally fixed this tool to make it not stupid
fi

# Push some media files to play
# (you'll probably need to configure this manually)
if [ x"$MEDIA" = xyes ]; then
  echo 'pushing music'
  pushd media/music
  for f in *.mp3 ; do adb push $f /home/phablet/Music ; done
  popd
  adb shell 'cd /home/phablet/Music && chown phablet.phablet *'

  echo 'pushing videos'
  pushd media/video
  # FIXME: rewrite this to be more generic
  #adb push ~/.cache/mwc/videos.tgz /home/phablet/Videos
  #echo 'extracting videos'
  #adb shell 'cd /home/phablet/Videos && tar xfvz videos.tgz && rm -f videos.tgz && chown phablet.phablet *'
  for f in * ; do adb push $f /home/phablet/Videos ; done
  popd
  adb shell 'cd /home/phablet/Videos && chown phablet.phablet *'

  # FIXME: rewrite this to be more generic
  echo 'pushing pictures'
  # FIXME: rewrite this to be more generic
  adb push ~/.cache/torches/torches.tar.gz /home/phablet/Pictures
  echo 'extracting pictures'
  adb shell 'cd /home/phablet/Pictures && tar xfvz torches.tar.gz && rm -f torches.tar.gz && chown phablet.phablet * .'
fi

# Maybe push some extra scripts?
#echo 'pushing scripts'
#rootcmd mount -o remount,rw /
#for f in $HOME/touch/scripts/* ; do
#  echo $f
#  adb push $f /home/phablet
#done
#rootcmd 'mount -o remount,ro /'

# Disable the welcome wizard
if [ x"$WIZARD" = x"no" ]; then
  echo 'disabling wizard'
  phablet-config welcome-wizard --disable
  adb wait-for-device
  REBOOT_NEEDED=yes
fi

# Disable the edges intro
if [ x"$EDGEINTRO" = x"no" ]; then
  echo 'disabling edges intro'
  phablet-config edges-intro --disable
  adb wait-for-device
fi

# force an apt-get update
if [ x"$APTUPDATE" = x"yes" ]; then
  echo 'apt-get update'
  rootcmd mount -o remount,rw /
  rootcmd apt-get update
  rootcmd mount -o remount,ro /
  REBOOT_NEEDED=yes
fi

if [ x"$REBOOT_NEEDED" = x"yes" ]; then
  echo 'rebooting to activate previous changes'
  adb reboot
  adb wait-for-device
fi

# All done, beep to notify the user.
if [ x"$BEEP" = x"yes" ]; then beep ; fi
